import sys
import typing
import bpy


def bake():
    '''Bake selected F-Curves to a set of sampled points defining a similar curve 

    '''

    pass


def clean(threshold: float = 0.001, channels: bool = False):
    '''Simplify F-Curves by removing closely spaced keyframes 

    :param threshold: Threshold 
    :type threshold: float
    :param channels: Channels 
    :type channels: bool
    '''

    pass


def click_insert(frame: float = 1.0, value: float = 1.0, extend: bool = False):
    '''Insert new keyframe at the cursor position for the active F-Curve 

    :param frame: Frame Number, Frame to insert keyframe on 
    :type frame: float
    :param value: Value, Value for keyframe on 
    :type value: float
    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    '''

    pass


def clickselect(extend: bool = False,
                deselect_all: bool = False,
                column: bool = False,
                curves: bool = False):
    '''Select keyframes by clicking on them 

    :param extend: Extend Select, Toggle keyframe selection instead of leaving newly selected keyframes only 
    :type extend: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor 
    :type deselect_all: bool
    :param column: Column Select, Select all keyframes that occur on the same frame as the one under the mouse 
    :type column: bool
    :param curves: Only Curves, Select all the keyframes in the curve 
    :type curves: bool
    '''

    pass


def copy():
    '''Copy selected keyframes to the copy/paste buffer 

    '''

    pass


def cursor_set(frame: float = 0.0, value: float = 0.0):
    '''Interactively set the current frame and value cursor 

    :param frame: Frame 
    :type frame: float
    :param value: Value 
    :type value: float
    '''

    pass


def delete():
    '''Remove all selected keyframes 

    '''

    pass


def driver_delete_invalid():
    '''Delete all visible drivers considered invalid 

    '''

    pass


def driver_variables_copy():
    '''Copy the driver variables of the active driver 

    '''

    pass


def driver_variables_paste(replace: bool = False):
    '''Add copied driver variables to the active driver 

    :param replace: Replace Existing, Replace existing driver variables, instead of just appending to the end of the existing list 
    :type replace: bool
    '''

    pass


def duplicate(mode: typing.Union[int, str] = 'TRANSLATION'):
    '''Make a copy of all selected keyframes 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    '''

    pass


def duplicate_move(GRAPH_OT_duplicate=None, TRANSFORM_OT_transform=None):
    '''Make a copy of all selected keyframes and move them 

    :param GRAPH_OT_duplicate: Duplicate Keyframes, Make a copy of all selected keyframes 
    :param TRANSFORM_OT_transform: Transform, Transform selected items by mode type 
    '''

    pass


def easing_type(type: typing.Union[int, str] = 'AUTO'):
    '''Set easing type for the F-Curve segments starting from the selected keyframes 

    :param type: TypeAUTO Automatic Easing, Easing type is chosen automatically based on what the type of interpolation used (e.g. ‘Ease In’ for transitional types, and ‘Ease Out’ for dynamic effects).EASE_IN Ease In, Only on the end closest to the next keyframe.EASE_OUT Ease Out, Only on the end closest to the first keyframe.EASE_IN_OUT Ease In and Out, Segment between both keyframes. 
    :type type: typing.Union[int, str]
    '''

    pass


def euler_filter():
    '''Fix large jumps and flips in the selected Euler Rotation F-Curves arising from rotation values being clipped when baking physics 

    '''

    pass


def extrapolation_type(type: typing.Union[int, str] = 'CONSTANT'):
    '''Set extrapolation mode for selected F-Curves 

    :param type: TypeCONSTANT Constant Extrapolation, Values on endpoint keyframes are held.LINEAR Linear Extrapolation, Straight-line slope of end segments are extended past the endpoint keyframes.MAKE_CYCLIC Make Cyclic (F-Modifier), Add Cycles F-Modifier if one doesn’t exist already.CLEAR_CYCLIC Clear Cyclic (F-Modifier), Remove Cycles F-Modifier if not needed anymore. 
    :type type: typing.Union[int, str]
    '''

    pass


def fmodifier_add(type: typing.Union[int, str] = 'NULL',
                  only_active: bool = True):
    '''Add F-Modifier to the active/selected F-Curves 

    :param type: TypeNULL Invalid.GENERATOR Generator, Generate a curve using a factorized or expanded polynomial.FNGENERATOR Built-In Function, Generate a curve using standard math functions such as sin and cos.ENVELOPE Envelope, Reshape F-Curve values - e.g. change amplitude of movements.CYCLES Cycles, Cyclic extend/repeat keyframe sequence.NOISE Noise, Add pseudo-random noise on top of F-Curves.LIMITS Limits, Restrict maximum and minimum values of F-Curve.STEPPED Stepped Interpolation, Snap values to nearest grid-step - e.g. for a stop-motion look. 
    :type type: typing.Union[int, str]
    :param only_active: Only Active, Only add F-Modifier to active F-Curve 
    :type only_active: bool
    '''

    pass


def fmodifier_copy():
    '''Copy the F-Modifier(s) of the active F-Curve 

    '''

    pass


def fmodifier_paste(only_active: bool = True, replace: bool = False):
    '''Add copied F-Modifiers to the selected F-Curves 

    :param only_active: Only Active, Only paste F-Modifiers on active F-Curve 
    :type only_active: bool
    :param replace: Replace Existing, Replace existing F-Modifiers, instead of just appending to the end of the existing list 
    :type replace: bool
    '''

    pass


def frame_jump():
    '''Place the cursor on the midpoint of selected keyframes 

    '''

    pass


def ghost_curves_clear():
    '''Clear F-Curve snapshots (Ghosts) for active Graph Editor 

    '''

    pass


def ghost_curves_create():
    '''Create snapshot (Ghosts) of selected F-Curves as background aid for active Graph Editor 

    '''

    pass


def handle_type(type: typing.Union[int, str] = 'FREE'):
    '''Set type of handle for selected keyframes 

    :param type: TypeFREE Free, Completely independent manually set handle.ALIGNED Aligned, Manually set handle with rotation locked together with its pair.VECTOR Vector, Automatic handles that create straight lines.AUTO Automatic, Automatic handles that create smooth curves.AUTO_CLAMPED Auto Clamped, Automatic handles that create smooth curves which only change direction at keyframes. 
    :type type: typing.Union[int, str]
    '''

    pass


def hide(unselected: bool = False):
    '''Hide selected curves from Graph Editor view 

    :param unselected: Unselected, Hide unselected rather than selected curves 
    :type unselected: bool
    '''

    pass


def interpolation_type(type: typing.Union[int, str] = 'CONSTANT'):
    '''Set interpolation mode for the F-Curve segments starting from the selected keyframes 

    :param type: TypeCONSTANT Constant, No interpolation, value of A gets held until B is encountered.LINEAR Linear, Straight-line interpolation between A and B (i.e. no ease in/out).BEZIER Bezier, Smooth interpolation between A and B, with some control over curve shape.SINE Sinusoidal, Sinusoidal easing (weakest, almost linear but with a slight curvature).QUAD Quadratic, Quadratic easing.CUBIC Cubic, Cubic easing.QUART Quartic, Quartic easing.QUINT Quintic, Quintic easing.EXPO Exponential, Exponential easing (dramatic).CIRC Circular, Circular easing (strongest and most dynamic).BACK Back, Cubic easing with overshoot and settle.BOUNCE Bounce, Exponentially decaying parabolic bounce, like when objects collide.ELASTIC Elastic, Exponentially decaying sine wave, like an elastic band. 
    :type type: typing.Union[int, str]
    '''

    pass


def keyframe_insert(type: typing.Union[int, str] = 'ALL'):
    '''Insert keyframes for the specified channels 

    :param type: TypeALL All Channels, Insert a keyframe on all visible and editable F-Curves using each curve’s current value.SEL Only Selected Channels, Insert a keyframe on selected F-Curves using each curve’s current value.CURSOR_ACTIVE Active Channels At Cursor, Insert a keyframe for the active F-Curve at the cursor point.CURSOR_SEL Selected Channels At Cursor, Insert a keyframe for selected F-Curves at the cursor point. 
    :type type: typing.Union[int, str]
    '''

    pass


def mirror(type: typing.Union[int, str] = 'CFRA'):
    '''Flip selected keyframes over the selected mirror line 

    :param type: TypeCFRA By Times over Current Frame, Flip times of selected keyframes using the current frame as the mirror line.VALUE By Values over Cursor Value, Flip values of selected keyframes using the cursor value (Y/Horizontal component) as the mirror line.YAXIS By Times over Time=0, Flip times of selected keyframes, effectively reversing the order they appear in.XAXIS By Values over Value=0, Flip values of selected keyframes (i.e. negative values become positive, and vice versa).MARKER By Times over First Selected Marker, Flip times of selected keyframes using the first selected marker as the reference point. 
    :type type: typing.Union[int, str]
    '''

    pass


def paste(offset: typing.Union[int, str] = 'START',
          merge: typing.Union[int, str] = 'MIX',
          flipped: bool = False):
    '''Paste keyframes from copy/paste buffer for the selected channels, starting on the current frame 

    :param offset: Offset, Paste time offset of keysSTART Frame Start, Paste keys starting at current frame.END Frame End, Paste keys ending at current frame.RELATIVE Frame Relative, Paste keys relative to the current frame when copying.NONE No Offset, Paste keys from original time. 
    :type offset: typing.Union[int, str]
    :param merge: Type, Method of merging pasted keys and existingMIX Mix, Overlay existing with new keys.OVER_ALL Overwrite All, Replace all keys.OVER_RANGE Overwrite Range, Overwrite keys in pasted range.OVER_RANGE_ALL Overwrite Entire Range, Overwrite keys in pasted range, using the range of all copied keys. 
    :type merge: typing.Union[int, str]
    :param flipped: Flipped, Paste keyframes from mirrored bones if they exist 
    :type flipped: bool
    '''

    pass


def previewrange_set():
    '''Automatically set Preview Range based on range of keyframes 

    '''

    pass


def reveal(select: bool = True):
    '''Make previously hidden curves visible again in Graph Editor view 

    :param select: Select 
    :type select: bool
    '''

    pass


def sample():
    '''Add keyframes on every frame between the selected keyframes 

    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Toggle selection of all keyframes 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_box(axis_range: bool = False,
               include_handles: bool = False,
               tweak: bool = False,
               xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET'):
    '''Select all keyframes within the specified region 

    :param axis_range: Axis Range 
    :type axis_range: bool
    :param include_handles: Include Handles, Are handles tested individually against the selection criteria 
    :type include_handles: bool
    :param tweak: Tweak, Operator has been activated using a tweak event 
    :type tweak: bool
    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_circle(x: int = 0,
                  y: int = 0,
                  radius: int = 25,
                  wait_for_input: bool = True,
                  mode: typing.Union[int, str] = 'SET'):
    '''Select keyframe points using circle selection 

    :param x: X 
    :type x: int
    :param y: Y 
    :type y: int
    :param radius: Radius 
    :type radius: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_column(mode: typing.Union[int, str] = 'KEYS'):
    '''Select all keyframes on the specified frame(s) 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_lasso(path: typing.List['bpy.types.OperatorMousePath'] = None,
                 mode: typing.Union[int, str] = 'SET'):
    '''Select keyframe points using lasso selection 

    :param path: Path 
    :type path: typing.List['bpy.types.OperatorMousePath']
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_leftright(mode: typing.Union[int, str] = 'CHECK',
                     extend: bool = False):
    '''Select keyframes to the left or the right of the current frame 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    :param extend: Extend Select 
    :type extend: bool
    '''

    pass


def select_less():
    '''Deselect keyframes on ends of selection islands 

    '''

    pass


def select_linked():
    '''Select keyframes occurring in the same F-Curves as selected ones 

    '''

    pass


def select_more():
    '''Select keyframes beside already selected ones 

    '''

    pass


def smooth():
    '''Apply weighted moving means to make selected F-Curves less bumpy 

    '''

    pass


def snap(type: typing.Union[int, str] = 'CFRA'):
    '''Snap selected keyframes to the chosen times/values 

    :param type: TypeCFRA Current Frame, Snap selected keyframes to the current frame.VALUE Cursor Value, Set values of selected keyframes to the cursor value (Y/Horizontal component).NEAREST_FRAME Nearest Frame, Snap selected keyframes to the nearest (whole) frame (use to fix accidental sub-frame offsets).NEAREST_SECOND Nearest Second, Snap selected keyframes to the nearest second.NEAREST_MARKER Nearest Marker, Snap selected keyframes to the nearest marker.HORIZONTAL Flatten Handles, Flatten handles for a smoother transition. 
    :type type: typing.Union[int, str]
    '''

    pass


def sound_bake(filepath: str = "",
               filter_blender: bool = False,
               filter_backup: bool = False,
               filter_image: bool = False,
               filter_movie: bool = True,
               filter_python: bool = False,
               filter_font: bool = False,
               filter_sound: bool = True,
               filter_text: bool = False,
               filter_btx: bool = False,
               filter_collada: bool = False,
               filter_alembic: bool = False,
               filter_folder: bool = True,
               filter_blenlib: bool = False,
               filemode: int = 9,
               show_multiview: bool = False,
               use_multiview: bool = False,
               display_type: typing.Union[int, str] = 'DEFAULT',
               sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA',
               low: float = 0.0,
               high: float = 100000.0,
               attack: float = 0.005,
               release: float = 0.2,
               threshold: float = 0.0,
               use_accumulate: bool = False,
               use_additive: bool = False,
               use_square: bool = False,
               sthreshold: float = 0.1):
    '''Bakes a sound wave to selected F-Curves 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param show_multiview: Enable Multi-View 
    :type show_multiview: bool
    :param use_multiview: Use Multi-View 
    :type use_multiview: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    :param low: Lowest frequency, Cutoff frequency of a high-pass filter that is applied to the audio data 
    :type low: float
    :param high: Highest frequency, Cutoff frequency of a low-pass filter that is applied to the audio data 
    :type high: float
    :param attack: Attack time, Value for the hull curve calculation that tells how fast the hull curve can rise (the lower the value the steeper it can rise) 
    :type attack: float
    :param release: Release time, Value for the hull curve calculation that tells how fast the hull curve can fall (the lower the value the steeper it can fall) 
    :type release: float
    :param threshold: Threshold, Minimum amplitude value needed to influence the hull curve 
    :type threshold: float
    :param use_accumulate: Accumulate, Only the positive differences of the hull curve amplitudes are summarized to produce the output 
    :type use_accumulate: bool
    :param use_additive: Additive, The amplitudes of the hull curve are summarized (or, when Accumulate is enabled, both positive and negative differences are accumulated) 
    :type use_additive: bool
    :param use_square: Square, The output is a square curve (negative values always result in -1, and positive ones in 1) 
    :type use_square: bool
    :param sthreshold: Square Threshold, Square only: all values with an absolute amplitude lower than that result in 0 
    :type sthreshold: float
    '''

    pass


def view_all(include_handles: bool = True):
    '''Reset viewable area to show full keyframe range 

    :param include_handles: Include Handles, Include handles of keyframes when calculating extents 
    :type include_handles: bool
    '''

    pass


def view_frame():
    '''Reset viewable area to show range around current frame 

    '''

    pass


def view_selected(include_handles: bool = True):
    '''Reset viewable area to show selected keyframe range 

    :param include_handles: Include Handles, Include handles of keyframes when calculating extents 
    :type include_handles: bool
    '''

    pass



import sys
import typing
import bpy


def brush_stroke(stroke: typing.List['bpy.types.OperatorStrokeElement'] = None,
                 mode: typing.Union[int, str] = 'NORMAL',
                 ignore_background_click: bool = False):
    '''Sculpt a stroke into the geometry 

    :param stroke: Stroke 
    :type stroke: typing.List['bpy.types.OperatorStrokeElement']
    :param mode: Stroke Mode, Action taken when a paint stroke is madeNORMAL Regular, Apply brush normally.INVERT Invert, Invert action of brush for duration of stroke.SMOOTH Smooth, Switch brush to smooth mode for duration of stroke. 
    :type mode: typing.Union[int, str]
    :param ignore_background_click: Ignore Background Click, Clicks on the background do not start the stroke 
    :type ignore_background_click: bool
    '''

    pass


def detail_flood_fill():
    '''Flood fill the mesh with the selected detail setting 

    '''

    pass


def dynamic_topology_toggle():
    '''Dynamic topology alters the mesh topology while sculpting 

    '''

    pass


def optimize():
    '''Recalculate the sculpt BVH to improve performance 

    '''

    pass


def sample_detail_size(location: int = (0, 0)):
    '''Sample the mesh detail on clicked point 

    :param location: Location, Screen Coordinates of sampling 
    :type location: int
    '''

    pass


def sculptmode_toggle():
    '''Toggle sculpt mode in 3D view 

    '''

    pass


def set_detail_size():
    '''Set the mesh detail (either relative or constant one, depending on current dyntopo mode) 

    '''

    pass


def set_persistent_base():
    '''Reset the copy of the mesh that is being sculpted on 

    '''

    pass


def symmetrize():
    '''Symmetrize the topology modifications 

    '''

    pass


def uv_sculpt_stroke(mode: typing.Union[int, str] = 'NORMAL'):
    '''Sculpt UVs using a brush 

    :param mode: Mode, Stroke ModeNORMAL Regular, Apply brush normally.INVERT Invert, Invert action of brush for duration of stroke.RELAX Relax, Switch brush to relax mode for duration of stroke. 
    :type mode: typing.Union[int, str]
    '''

    pass



import sys
import typing
import bmesh


def edge_rotate(edge: 'bmesh.types.BMEdge',
                ccw: bool = False) -> 'bmesh.types.BMEdge':
    '''Rotate the edge and return the newly created edge. If rotating the edge fails, None will be returned. 

    :param edge: The edge to rotate. 
    :type edge: 'bmesh.types.BMEdge'
    :param ccw: When True the edge will be rotated counter clockwise. 
    :type ccw: bool
    :return:  The newly rotated edge. 
    '''

    pass


def edge_split(edge: 'bmesh.types.BMEdge', vert: 'bmesh.types.BMVert',
               fac: float):
    '''Split an edge, return the newly created data. 

    :param edge: The edge to split. 
    :type edge: 'bmesh.types.BMEdge'
    :param vert: One of the verts on the edge, defines the split direction. 
    :type vert: 'bmesh.types.BMVert'
    :param fac: The point on the edge where the new vert will be created [0 - 1]. 
    :type fac: float
    :return:  The newly created (edge, vert) pair. 
    '''

    pass


def face_flip(faces):
    '''Flip the faces direction. 

    :param face: Face to flip. 
    :type face: 'bmesh.types.BMFace'
    '''

    pass


def face_join(faces: 'bmesh.types.BMFace',
              remove: bool = True) -> 'bmesh.types.BMFace':
    '''Joins a sequence of faces. 

    :param faces: Sequence of faces. 
    :type faces: 'bmesh.types.BMFace'
    :param remove: Remove the edges and vertices between the faces. 
    :type remove: bool
    :return:  The newly created face or None on failure. 
    '''

    pass


def face_split(face: 'bmesh.types.BMFace',
               vert_a: 'bmesh.types.BMVert',
               vert_b: 'bmesh.types.BMVert',
               coords: typing.List[float] = (),
               use_exist: bool = True,
               example: 'bmesh.types.BMEdge' = None) -> 'bmesh.types.BMFace':
    '''Face split with optional intermediate points. 

    :param face: The face to cut. 
    :type face: 'bmesh.types.BMFace'
    :param vert_a: First vertex to cut in the face (face must contain the vert). 
    :type vert_a: 'bmesh.types.BMVert'
    :param vert_b: Second vertex to cut in the face (face must contain the vert). 
    :type vert_b: 'bmesh.types.BMVert'
    :param coords: Optional argument to define points in between vert_a and vert_b. 
    :type coords: typing.List[float]
    :param use_exist: .Use an existing edge if it exists (Only used when coords argument is empty or omitted) 
    :type use_exist: bool
    :param example: Newly created edge will copy settings from this one. 
    :type example: 'bmesh.types.BMEdge'
    :return:  The newly created face or None on failure. 
    '''

    pass


def face_split_edgenet(face: 'bmesh.types.BMFace',
                       edgenet: 'bmesh.types.BMEdge') -> 'bmesh.types.BMFace':
    '''Splits a face into any number of regions defined by an edgenet. 

    :param face: The face to split. 
    :type face: 'bmesh.types.BMFace'
    :param face: The face to split. 
    :param edgenet: Sequence of edges. 
    :type edgenet: 'bmesh.types.BMEdge'
    :return:  The newly created faces. 
    '''

    pass


def face_vert_separate(face: 'bmesh.types.BMFace', vert: 'bmesh.types.BMVert'):
    '''Rip a vertex in a face away and add a new vertex. 

    :param face: The face to separate. 
    :type face: 'bmesh.types.BMFace'
    :param vert: A vertex in the face to separate. 
    :type vert: 'bmesh.types.BMVert'
    '''

    pass


def loop_separate(loop: 'bmesh.types.BMLoop'):
    '''Rip a vertex in a face away and add a new vertex. 

    :param loop: The loop to separate. 
    :type loop: 'bmesh.types.BMLoop'
    '''

    pass


def vert_collapse_edge(vert: 'bmesh.types.BMVert',
                       edge: 'bmesh.types.BMEdge') -> 'bmesh.types.BMEdge':
    '''Collapse a vertex into an edge. 

    :param vert: The vert that will be collapsed. 
    :type vert: 'bmesh.types.BMVert'
    :param edge: The edge to collapse into. 
    :type edge: 'bmesh.types.BMEdge'
    :return:  The resulting edge from the collapse operation. 
    '''

    pass


def vert_collapse_faces(vert: 'bmesh.types.BMVert', edge: 'bmesh.types.BMEdge',
                        fac: float, join_faces) -> 'bmesh.types.BMEdge':
    '''Collapses a vertex that has only two manifold edges onto a vertex it shares an edge with. 

    :param vert: The vert that will be collapsed. 
    :type vert: 'bmesh.types.BMVert'
    :param edge: The edge to collapse into. 
    :type edge: 'bmesh.types.BMEdge'
    :param fac: The factor to use when merging customdata [0 - 1]. 
    :type fac: float
    :return:  The resulting edge from the collapse operation. 
    '''

    pass


def vert_dissolve(vert: 'bmesh.types.BMVert') -> bool:
    '''Dissolve this vertex (will be removed). 

    :param vert: The vert to be dissolved. 
    :type vert: 'bmesh.types.BMVert'
    :return:  True when the vertex dissolve is successful. 
    '''

    pass


def vert_separate(vert: 'bmesh.types.BMVert',
                  edges: 'bmesh.types.BMEdge') -> 'bmesh.types.BMVert':
    '''Separate this vertex at every edge. 

    :param vert: The vert to be separated. 
    :type vert: 'bmesh.types.BMVert'
    :param edges: The edges to separated. 
    :type edges: 'bmesh.types.BMEdge'
    :return:  The newly separated verts (including the vertex passed). 
    '''

    pass


def vert_splice(vert: 'bmesh.types.BMVert', vert_target: 'bmesh.types.BMVert'):
    '''Splice vert into vert_target. 

    :param vert: The vertex to be removed. 
    :type vert: 'bmesh.types.BMVert'
    :param vert_target: The vertex to use. 
    :type vert_target: 'bmesh.types.BMVert'
    '''

    pass



import sys
import typing


def add():
    '''Add new cache 

    '''

    pass


def bake(bake: bool = False):
    '''Bake physics 

    :param bake: Bake 
    :type bake: bool
    '''

    pass


def bake_all(bake: bool = True):
    '''Bake all physics 

    :param bake: Bake 
    :type bake: bool
    '''

    pass


def bake_from_cache():
    '''Bake from cache 

    '''

    pass


def free_bake():
    '''Delete physics bake 

    '''

    pass


def free_bake_all():
    '''Delete all baked caches of all objects in the current scene 

    '''

    pass


def remove():
    '''Delete current cache 

    '''

    pass



import sys
import typing


def align():
    '''Align selected bones to the active bone (or to their parent) 

    '''

    pass


def armature_layers(
        layers: bool = (False, False, False, False, False, False, False, False,
                        False, False, False, False, False, False, False, False,
                        False, False, False, False, False, False, False, False,
                        False, False, False, False, False, False, False,
                        False)):
    '''Change the visible armature layers 

    :param layers: Layer, Armature layers to make visible 
    :type layers: bool
    '''

    pass


def autoside_names(type: typing.Union[int, str] = 'XAXIS'):
    '''Automatically renames the selected bones according to which side of the target axis they fall on 

    :param type: Axis, Axis tag names withXAXIS X-Axis, Left/Right.YAXIS Y-Axis, Front/Back.ZAXIS Z-Axis, Top/Bottom. 
    :type type: typing.Union[int, str]
    '''

    pass


def bone_layers(
        layers: bool = (False, False, False, False, False, False, False, False,
                        False, False, False, False, False, False, False, False,
                        False, False, False, False, False, False, False, False,
                        False, False, False, False, False, False, False,
                        False)):
    '''Change the layers that the selected bones belong to 

    :param layers: Layer, Armature layers that bone belongs to 
    :type layers: bool
    '''

    pass


def bone_primitive_add(name: str = "Bone"):
    '''Add a new bone located at the 3D-Cursor 

    :param name: Name, Name of the newly created bone 
    :type name: str
    '''

    pass


def calculate_roll(type: typing.Union[int, str] = 'POS_X',
                   axis_flip: bool = False,
                   axis_only: bool = False):
    '''Automatically fix alignment of select bones’ axes 

    :param type: Type 
    :type type: typing.Union[int, str]
    :param axis_flip: Flip Axis, Negate the alignment axis 
    :type axis_flip: bool
    :param axis_only: Shortest Rotation, Ignore the axis direction, use the shortest rotation to align 
    :type axis_only: bool
    '''

    pass


def click_extrude():
    '''Create a new bone going from the last selected joint to the mouse position 

    '''

    pass


def delete():
    '''Remove selected bones from the armature 

    '''

    pass


def dissolve():
    '''Dissolve selected bones from the armature 

    '''

    pass


def duplicate(do_flip_names: bool = False):
    '''Make copies of the selected bones within the same armature 

    :param do_flip_names: Flip Names, Try to flip names of the bones, if possible, instead of adding a number extension 
    :type do_flip_names: bool
    '''

    pass


def duplicate_move(ARMATURE_OT_duplicate=None, TRANSFORM_OT_translate=None):
    '''Make copies of the selected bones within the same armature and move them 

    :param ARMATURE_OT_duplicate: Duplicate Selected Bone(s), Make copies of the selected bones within the same armature 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def extrude(forked: bool = False):
    '''Create new bones from the selected joints 

    :param forked: Forked 
    :type forked: bool
    '''

    pass


def extrude_forked(ARMATURE_OT_extrude=None, TRANSFORM_OT_translate=None):
    '''Create new bones from the selected joints and move them 

    :param ARMATURE_OT_extrude: Extrude, Create new bones from the selected joints 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def extrude_move(ARMATURE_OT_extrude=None, TRANSFORM_OT_translate=None):
    '''Create new bones from the selected joints and move them 

    :param ARMATURE_OT_extrude: Extrude, Create new bones from the selected joints 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def fill():
    '''Add bone between selected joint(s) and/or 3D-Cursor 

    '''

    pass


def flip_names(do_strip_numbers: bool = False):
    '''Flips (and corrects) the axis suffixes of the names of selected bones 

    :param do_strip_numbers: Strip Numbers, Try to remove right-most dot-number from flipped names (WARNING: may result in incoherent naming in some cases) 
    :type do_strip_numbers: bool
    '''

    pass


def hide(unselected: bool = False):
    '''Tag selected bones to not be visible in Edit Mode 

    :param unselected: Unselected, Hide unselected rather than selected 
    :type unselected: bool
    '''

    pass


def layers_show_all(all: bool = True):
    '''Make all armature layers visible 

    :param all: All Layers, Enable all layers or just the first 16 (top row) 
    :type all: bool
    '''

    pass


def merge(type: typing.Union[int, str] = 'WITHIN_CHAIN'):
    '''Merge continuous chains of selected bones 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def parent_clear(type: typing.Union[int, str] = 'CLEAR'):
    '''Remove the parent-child relationship between selected bones and their parents 

    :param type: ClearType, What way to clear parenting 
    :type type: typing.Union[int, str]
    '''

    pass


def parent_set(type: typing.Union[int, str] = 'CONNECTED'):
    '''Set the active bone as the parent of the selected bones 

    :param type: ParentType, Type of parenting 
    :type type: typing.Union[int, str]
    '''

    pass


def reveal(select: bool = True):
    '''Reveal all bones hidden in Edit Mode 

    :param select: Select 
    :type select: bool
    '''

    pass


def roll_clear(roll: float = 0.0):
    '''Clear roll for selected bones 

    :param roll: Roll 
    :type roll: float
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Toggle selection status of all bones 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_hierarchy(direction: typing.Union[int, str] = 'PARENT',
                     extend: bool = False):
    '''Select immediate parent/children of selected bones 

    :param direction: Direction 
    :type direction: typing.Union[int, str]
    :param extend: Extend, Extend the selection 
    :type extend: bool
    '''

    pass


def select_less():
    '''Deselect those bones at the boundary of each selection region 

    '''

    pass


def select_linked(deselect: bool = False):
    '''Select bones related to selected ones by parent/child relationships 

    :param deselect: Deselect 
    :type deselect: bool
    '''

    pass


def select_mirror(only_active: bool = False, extend: bool = False):
    '''Mirror the bone selection 

    :param only_active: Active Only, Only operate on the active bone 
    :type only_active: bool
    :param extend: Extend, Extend the selection 
    :type extend: bool
    '''

    pass


def select_more():
    '''Select those bones connected to the initial selection 

    '''

    pass


def select_similar(type: typing.Union[int, str] = 'LENGTH',
                   threshold: float = 0.1):
    '''Select similar bones by property types 

    :param type: Type 
    :type type: typing.Union[int, str]
    :param threshold: Threshold 
    :type threshold: float
    '''

    pass


def separate():
    '''Isolate selected bones into a separate armature 

    '''

    pass


def shortest_path_pick():
    '''Select shortest path between two bones 

    '''

    pass


def split():
    '''Split off selected bones from connected unselected bones 

    '''

    pass


def subdivide(number_cuts: int = 1):
    '''Break selected bones into chains of smaller bones 

    :param number_cuts: Number of Cuts 
    :type number_cuts: int
    '''

    pass


def switch_direction():
    '''Change the direction that a chain of bones points in (head <-> tail swap) 

    '''

    pass


def symmetrize(direction: typing.Union[int, str] = 'NEGATIVE_X'):
    '''Enforce symmetry, make copies of the selection or use existing 

    :param direction: Direction, Which sides to copy from and to (when both are selected) 
    :type direction: typing.Union[int, str]
    '''

    pass



import sys
import typing
import mathutils


def draw_circle_2d(position: 'mathutils.Vector',
                   color,
                   radius: float,
                   segments: int = 32):
    '''Draw a circle. 

    :param position: Position where the circle will be drawn. 
    :type position: 'mathutils.Vector'
    :param color: Color of the circle. To use transparency GL_BLEND has to be enabled. 
    :param radius: Radius of the circle. 
    :type radius: float
    :param segments: How many segments will be used to draw the circle. Higher values give besser results but the drawing will take longer. 
    :type segments: int
    '''

    pass


def draw_texture_2d(texture_id: int, position: 'mathutils.Vector',
                    width: float, height: float):
    '''Draw a 2d texture. 

    :param texture_id: OpenGL id of the texture (e.g. bpy.types.Image.bindcode). 
    :type texture_id: int
    :param position: Position of the lower left corner. 
    :type position: 'mathutils.Vector'
    :param width: Width of the image when drawn (not necessarily the original width of the texture). 
    :type width: float
    :param height: Height of the image when drawn. 
    :type height: float
    '''

    pass



import sys
import typing


def report_copy():
    '''Copy selected reports to Clipboard 

    '''

    pass


def report_delete():
    '''Delete selected reports 

    '''

    pass


def report_replay():
    '''Replay selected reports 

    '''

    pass


def reports_display_update():
    '''Update the display of reports in Blender UI (internal use) 

    '''

    pass


def select_all(action: typing.Union[int, str] = 'SELECT'):
    '''Change selection of all visible reports 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET'):
    '''Toggle box selection 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_pick(report_index: int = 0, extend: bool = False):
    '''Select reports by index 

    :param report_index: Report, Index of the report 
    :type report_index: int
    :param extend: Extend, Extend report selection 
    :type extend: bool
    '''

    pass



import sys
import typing
import bpy


def background_image_add(
        name: str = "Image",
        filepath: str = "",
        filter_blender: bool = False,
        filter_backup: bool = False,
        filter_image: bool = True,
        filter_movie: bool = True,
        filter_python: bool = False,
        filter_font: bool = False,
        filter_sound: bool = False,
        filter_text: bool = False,
        filter_btx: bool = False,
        filter_collada: bool = False,
        filter_alembic: bool = False,
        filter_folder: bool = True,
        filter_blenlib: bool = False,
        filemode: int = 9,
        relative_path: bool = True,
        show_multiview: bool = False,
        use_multiview: bool = False,
        display_type: typing.Union[int, str] = 'DEFAULT',
        sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA'):
    '''Add a new background image 

    :param name: Name, Image name to assign 
    :type name: str
    :param filepath: File Path, Path to file 
    :type filepath: str
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param show_multiview: Enable Multi-View 
    :type show_multiview: bool
    :param use_multiview: Use Multi-View 
    :type use_multiview: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    '''

    pass


def background_image_remove(index: int = 0):
    '''Remove a background image from the 3D view 

    :param index: Index, Background image index to remove 
    :type index: int
    '''

    pass


def camera_to_view():
    '''Set camera view to active view 

    '''

    pass


def camera_to_view_selected():
    '''Move the camera so selected objects are framed 

    '''

    pass


def clear_render_border():
    '''Clear the boundaries of the border render and disable border render 

    '''

    pass


def clip_border(xmin: int = 0,
                xmax: int = 0,
                ymin: int = 0,
                ymax: int = 0,
                wait_for_input: bool = True):
    '''Set the view clipping region 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    '''

    pass


def copybuffer():
    '''Selected objects are copied to the clipboard 

    '''

    pass


def cursor3d(use_depth: bool = True,
             orientation: typing.Union[int, str] = 'VIEW'):
    '''Set the location of the 3D cursor 

    :param use_depth: Surface Project, Project onto the surface 
    :type use_depth: bool
    :param orientation: Orientation, Preset viewpoint to useNONE None, Leave orientation unchanged.VIEW View, Orient to the viewport.XFORM Transform, Orient to the current transform setting.GEOM Geometry, Match the surface normal. 
    :type orientation: typing.Union[int, str]
    '''

    pass


def dolly(mx: int = 0,
          my: int = 0,
          delta: int = 0,
          use_cursor_init: bool = True):
    '''Dolly in/out in the view 

    :param mx: Region Position X 
    :type mx: int
    :param my: Region Position Y 
    :type my: int
    :param delta: Delta 
    :type delta: int
    :param use_cursor_init: Cursor Init, Use initial cursor position 
    :type use_cursor_init: bool
    '''

    pass


def edit_mesh_extrude_individual_move():
    '''Extrude each individual face separately along local normals 

    '''

    pass


def edit_mesh_extrude_move_normal():
    '''Extrude region together along the average normal 

    '''

    pass


def edit_mesh_extrude_move_shrink_fatten():
    '''Extrude region together along local normals 

    '''

    pass


def fly():
    '''Interactively fly around the scene 

    '''

    pass


def localview(frame_selected: bool = True):
    '''Toggle display of selected object(s) separately and centered in view 

    :param frame_selected: Frame Selected, Move the view to frame the selected objects 
    :type frame_selected: bool
    '''

    pass


def localview_remove_from():
    '''Move selected objects out of local view 

    '''

    pass


def move(use_cursor_init: bool = True):
    '''Move the view 

    :param use_cursor_init: Cursor Init, Use initial cursor position 
    :type use_cursor_init: bool
    '''

    pass


def navigate():
    '''Interactively navigate around the scene (uses the mode (walk/fly) preference) 

    '''

    pass


def ndof_all():
    '''Pan and rotate the view with the 3D mouse 

    '''

    pass


def ndof_orbit():
    '''Orbit the view using the 3D mouse 

    '''

    pass


def ndof_orbit_zoom():
    '''Orbit and zoom the view using the 3D mouse 

    '''

    pass


def ndof_pan():
    '''Pan the view with the 3D mouse 

    '''

    pass


def object_as_camera():
    '''Set the active object as the active camera for this view or scene 

    '''

    pass


def object_mode_pie_or_toggle():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def pastebuffer(autoselect: bool = True, active_collection: bool = True):
    '''Objects from the clipboard are pasted 

    :param autoselect: Select, Select pasted objects 
    :type autoselect: bool
    :param active_collection: Active Collection, Put pasted objects in the active collection 
    :type active_collection: bool
    '''

    pass


def render_border(xmin: int = 0,
                  xmax: int = 0,
                  ymin: int = 0,
                  ymax: int = 0,
                  wait_for_input: bool = True):
    '''Set the boundaries of the border render and enable border render 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    '''

    pass


def rotate(use_cursor_init: bool = True):
    '''Rotate the view 

    :param use_cursor_init: Cursor Init, Use initial cursor position 
    :type use_cursor_init: bool
    '''

    pass


def ruler_add():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def ruler_remove():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def select(extend: bool = False,
           deselect: bool = False,
           toggle: bool = False,
           deselect_all: bool = False,
           center: bool = False,
           enumerate: bool = False,
           object: bool = False,
           location: int = (0, 0)):
    '''Select and activate item(s) 

    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    :param deselect: Deselect, Remove from selection 
    :type deselect: bool
    :param toggle: Toggle Selection, Toggle the selection 
    :type toggle: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor 
    :type deselect_all: bool
    :param center: Center, Use the object center when selecting, in editmode used to extend object selection 
    :type center: bool
    :param enumerate: Enumerate, List objects under the mouse (object mode only) 
    :type enumerate: bool
    :param object: Object, Use object selection (editmode only) 
    :type object: bool
    :param location: Location, Mouse location 
    :type location: int
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET'):
    '''Select items using box selection 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection.XOR Difference, Inverts existing selection.AND Intersect, Intersect existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_circle(x: int = 0,
                  y: int = 0,
                  radius: int = 25,
                  wait_for_input: bool = True,
                  mode: typing.Union[int, str] = 'SET'):
    '''Select items using circle selection 

    :param x: X 
    :type x: int
    :param y: Y 
    :type y: int
    :param radius: Radius 
    :type radius: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_lasso(path: typing.List['bpy.types.OperatorMousePath'] = None,
                 mode: typing.Union[int, str] = 'SET'):
    '''Select items using lasso selection 

    :param path: Path 
    :type path: typing.List['bpy.types.OperatorMousePath']
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection.XOR Difference, Inverts existing selection.AND Intersect, Intersect existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_menu(name: typing.Union[int, str] = '',
                extend: bool = False,
                deselect: bool = False,
                toggle: bool = False):
    '''Menu object selection 

    :param name: Object Name 
    :type name: typing.Union[int, str]
    :param extend: Extend 
    :type extend: bool
    :param deselect: Deselect 
    :type deselect: bool
    :param toggle: Toggle 
    :type toggle: bool
    '''

    pass


def smoothview():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def snap_cursor_to_active():
    '''Snap 3D cursor to the active item 

    '''

    pass


def snap_cursor_to_center():
    '''Snap 3D cursor to the world origin 

    '''

    pass


def snap_cursor_to_grid():
    '''Snap 3D cursor to the nearest grid division 

    '''

    pass


def snap_cursor_to_selected():
    '''Snap 3D cursor to the middle of the selected item(s) 

    '''

    pass


def snap_selected_to_active():
    '''Snap selected item(s) to the active item 

    '''

    pass


def snap_selected_to_cursor(use_offset: bool = True):
    '''Snap selected item(s) to the 3D cursor 

    :param use_offset: Offset, If the selection should be snapped as a whole or by each object center 
    :type use_offset: bool
    '''

    pass


def snap_selected_to_grid():
    '''Snap selected item(s) to their nearest grid division 

    '''

    pass


def toggle_matcap_flip():
    '''Flip MatCap 

    '''

    pass


def toggle_shading(type: typing.Union[int, str] = 'WIREFRAME'):
    '''Toggle shading type in 3D viewport 

    :param type: Type, Shading type to toggleWIREFRAME Wireframe, Toggle wireframe shading.SOLID Solid, Toggle solid shading.MATERIAL LookDev, Toggle lookdev shading.RENDERED Rendered, Toggle rendered shading. 
    :type type: typing.Union[int, str]
    '''

    pass


def toggle_xray():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def transform_gizmo_set(extend: bool = False,
                        type: typing.Set[typing.Union[int, str]] = {}):
    '''Set the current transform gizmo 

    :param extend: extend 
    :type extend: bool
    :param type: type 
    :type type: typing.Set[typing.Union[int, str]]
    '''

    pass


def view_all(use_all_regions: bool = False, center: bool = False):
    '''View all objects in scene 

    :param use_all_regions: All Regions, View selected for all regions 
    :type use_all_regions: bool
    :param center: Center 
    :type center: bool
    '''

    pass


def view_axis(type: typing.Union[int, str] = 'LEFT',
              align_active: bool = False,
              relative: bool = False):
    '''Use a preset viewpoint 

    :param type: View, Preset viewpoint to useLEFT Left, View From the Left.RIGHT Right, View From the Right.BOTTOM Bottom, View From the Bottom.TOP Top, View From the Top.FRONT Front, View From the Front.BACK Back, View From the Back. 
    :type type: typing.Union[int, str]
    :param align_active: Align Active, Align to the active object’s axis 
    :type align_active: bool
    :param relative: Relative, Rotate relative to the current orientation 
    :type relative: bool
    '''

    pass


def view_camera():
    '''Toggle the camera view 

    '''

    pass


def view_center_camera():
    '''Center the camera view 

    '''

    pass


def view_center_cursor():
    '''Center the view so that the cursor is in the middle of the view 

    '''

    pass


def view_center_lock():
    '''Center the view lock offset 

    '''

    pass


def view_center_pick():
    '''Center the view to the Z-depth position under the mouse cursor 

    '''

    pass


def view_lock_clear():
    '''Clear all view locking 

    '''

    pass


def view_lock_to_active():
    '''Lock the view to the active object/bone 

    '''

    pass


def view_orbit(angle: float = 0.0, type: typing.Union[int, str] = 'ORBITLEFT'):
    '''Orbit the view 

    :param angle: Roll 
    :type angle: float
    :param type: Orbit, Direction of View OrbitORBITLEFT Orbit Left, Orbit the view around to the Left.ORBITRIGHT Orbit Right, Orbit the view around to the Right.ORBITUP Orbit Up, Orbit the view Up.ORBITDOWN Orbit Down, Orbit the view Down. 
    :type type: typing.Union[int, str]
    '''

    pass


def view_pan(type: typing.Union[int, str] = 'PANLEFT'):
    '''Pan the view in a given direction 

    :param type: Pan, Direction of View PanPANLEFT Pan Left, Pan the view to the Left.PANRIGHT Pan Right, Pan the view to the Right.PANUP Pan Up, Pan the view Up.PANDOWN Pan Down, Pan the view Down. 
    :type type: typing.Union[int, str]
    '''

    pass


def view_persportho():
    '''Switch the current view from perspective/orthographic projection 

    '''

    pass


def view_roll(angle: float = 0.0, type: typing.Union[int, str] = 'ANGLE'):
    '''Roll the view 

    :param angle: Roll 
    :type angle: float
    :param type: Roll Angle Source, How roll angle is calculatedANGLE Roll Angle, Roll the view using an angle value.LEFT Roll Left, Roll the view around to the Left.RIGHT Roll Right, Roll the view around to the Right. 
    :type type: typing.Union[int, str]
    '''

    pass


def view_selected(use_all_regions: bool = False):
    '''Move the view to the selection center 

    :param use_all_regions: All Regions, View selected for all regions 
    :type use_all_regions: bool
    '''

    pass


def walk():
    '''Interactively walk around the scene 

    '''

    pass


def zoom(mx: int = 0,
         my: int = 0,
         delta: int = 0,
         use_cursor_init: bool = True):
    '''Zoom in/out in the view 

    :param mx: Region Position X 
    :type mx: int
    :param my: Region Position Y 
    :type my: int
    :param delta: Delta 
    :type delta: int
    :param use_cursor_init: Cursor Init, Use initial cursor position 
    :type use_cursor_init: bool
    '''

    pass


def zoom_border(xmin: int = 0,
                xmax: int = 0,
                ymin: int = 0,
                ymax: int = 0,
                wait_for_input: bool = True,
                zoom_out: bool = False):
    '''Zoom in the view to the nearest object contained in the border 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param zoom_out: Zoom Out 
    :type zoom_out: bool
    '''

    pass


def zoom_camera_1_to_1():
    '''Match the camera to 1:1 to the render output 

    '''

    pass



import sys
import typing
import bpy


def add_feather_vertex(location: float = (0.0, 0.0)):
    '''Add vertex to feather 

    :param location: Location, Location of vertex in normalized space 
    :type location: float
    '''

    pass


def add_feather_vertex_slide(MASK_OT_add_feather_vertex=None,
                             MASK_OT_slide_point=None):
    '''Add new vertex to feather and slide it 

    :param MASK_OT_add_feather_vertex: Add Feather Vertex, Add vertex to feather 
    :param MASK_OT_slide_point: Slide Point, Slide control points 
    '''

    pass


def add_vertex(location: float = (0.0, 0.0)):
    '''Add vertex to active spline 

    :param location: Location, Location of vertex in normalized space 
    :type location: float
    '''

    pass


def add_vertex_slide(MASK_OT_add_vertex=None, MASK_OT_slide_point=None):
    '''Add new vertex and slide it 

    :param MASK_OT_add_vertex: Add Vertex, Add vertex to active spline 
    :param MASK_OT_slide_point: Slide Point, Slide control points 
    '''

    pass


def copy_splines():
    '''Copy selected splines to clipboard 

    '''

    pass


def cyclic_toggle():
    '''Toggle cyclic for selected splines 

    '''

    pass


def delete():
    '''Delete selected control points or splines 

    '''

    pass


def duplicate():
    '''Duplicate selected control points and segments between them 

    '''

    pass


def duplicate_move(MASK_OT_duplicate=None, TRANSFORM_OT_translate=None):
    '''Duplicate mask and move 

    :param MASK_OT_duplicate: Duplicate Mask, Duplicate selected control points and segments between them 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def feather_weight_clear():
    '''Reset the feather weight to zero 

    '''

    pass


def handle_type_set(type: typing.Union[int, str] = 'AUTO'):
    '''Set type of handles for selected control points 

    :param type: Type, Spline type 
    :type type: typing.Union[int, str]
    '''

    pass


def hide_view_clear(select: bool = True):
    '''Reveal the layer by setting the hide flag 

    :param select: Select 
    :type select: bool
    '''

    pass


def hide_view_set(unselected: bool = False):
    '''Hide the layer by setting the hide flag 

    :param unselected: Unselected, Hide unselected rather than selected layers 
    :type unselected: bool
    '''

    pass


def layer_move(direction: typing.Union[int, str] = 'UP'):
    '''Move the active layer up/down in the list 

    :param direction: Direction, Direction to move the active layer 
    :type direction: typing.Union[int, str]
    '''

    pass


def layer_new(name: str = ""):
    '''Add new mask layer for masking 

    :param name: Name, Name of new mask layer 
    :type name: str
    '''

    pass


def layer_remove():
    '''Remove mask layer 

    '''

    pass


def new(name: str = ""):
    '''Create new mask 

    :param name: Name, Name of new mask 
    :type name: str
    '''

    pass


def normals_make_consistent():
    '''Re-calculate the direction of selected handles 

    '''

    pass


def parent_clear():
    '''Clear the mask’s parenting 

    '''

    pass


def parent_set():
    '''Set the mask’s parenting 

    '''

    pass


def paste_splines():
    '''Paste splines from clipboard 

    '''

    pass


def primitive_circle_add(size: float = 100.0, location: float = (0.0, 0.0)):
    '''Add new circle-shaped spline 

    :param size: Size, Size of new circle 
    :type size: float
    :param location: Location, Location of new circle 
    :type location: float
    '''

    pass


def primitive_square_add(size: float = 100.0, location: float = (0.0, 0.0)):
    '''Add new square-shaped spline 

    :param size: Size, Size of new circle 
    :type size: float
    :param location: Location, Location of new circle 
    :type location: float
    '''

    pass


def select(extend: bool = False,
           deselect: bool = False,
           toggle: bool = False,
           deselect_all: bool = False,
           location: float = (0.0, 0.0)):
    '''Select spline points 

    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    :param deselect: Deselect, Remove from selection 
    :type deselect: bool
    :param toggle: Toggle Selection, Toggle the selection 
    :type toggle: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor 
    :type deselect_all: bool
    :param location: Location, Location of vertex in normalized space 
    :type location: float
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Change selection of all curve points 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET'):
    '''Select curve points using box selection 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_circle(x: int = 0,
                  y: int = 0,
                  radius: int = 25,
                  wait_for_input: bool = True,
                  mode: typing.Union[int, str] = 'SET'):
    '''Select curve points using circle selection 

    :param x: X 
    :type x: int
    :param y: Y 
    :type y: int
    :param radius: Radius 
    :type radius: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_lasso(path: typing.List['bpy.types.OperatorMousePath'] = None,
                 mode: typing.Union[int, str] = 'SET'):
    '''Select curve points using lasso selection 

    :param path: Path 
    :type path: typing.List['bpy.types.OperatorMousePath']
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_less():
    '''Deselect spline points at the boundary of each selection region 

    '''

    pass


def select_linked():
    '''Select all curve points linked to already selected ones 

    '''

    pass


def select_linked_pick(deselect: bool = False):
    '''(De)select all points linked to the curve under the mouse cursor 

    :param deselect: Deselect 
    :type deselect: bool
    '''

    pass


def select_more():
    '''Select more spline points connected to initial selection 

    '''

    pass


def shape_key_clear():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def shape_key_feather_reset():
    '''Reset feather weights on all selected points animation values 

    '''

    pass


def shape_key_insert():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def shape_key_rekey(location: bool = True, feather: bool = True):
    '''Recalculate animation data on selected points for frames selected in the dopesheet 

    :param location: Location 
    :type location: bool
    :param feather: Feather 
    :type feather: bool
    '''

    pass


def slide_point(slide_feather: bool = False, is_new_point: bool = False):
    '''Slide control points 

    :param slide_feather: Slide Feather, First try to slide feather instead of vertex 
    :type slide_feather: bool
    :param is_new_point: Slide New Point, Newly created vertex is being slid 
    :type is_new_point: bool
    '''

    pass


def slide_spline_curvature():
    '''Slide a point on the spline to define it’s curvature 

    '''

    pass


def switch_direction():
    '''Switch direction of selected splines 

    '''

    pass



import sys
import typing


def rule_add(type: typing.Union[int, str] = 'GOAL'):
    '''Add a boid rule to the current boid state 

    :param type: TypeGOAL Goal, Go to assigned object or loudest assigned signal source.AVOID Avoid, Get away from assigned object or loudest assigned signal source.AVOID_COLLISION Avoid Collision, Maneuver to avoid collisions with other boids and deflector objects in near future.SEPARATE Separate, Keep from going through other boids.FLOCK Flock, Move to center of neighbors and match their velocity.FOLLOW_LEADER Follow Leader, Follow a boid or assigned object.AVERAGE_SPEED Average Speed, Maintain speed, flight level or wander.FIGHT Fight, Go to closest enemy and attack when in range. 
    :type type: typing.Union[int, str]
    '''

    pass


def rule_del():
    '''Delete current boid rule 

    '''

    pass


def rule_move_down():
    '''Move boid rule down in the list 

    '''

    pass


def rule_move_up():
    '''Move boid rule up in the list 

    '''

    pass


def state_add():
    '''Add a boid state to the particle system 

    '''

    pass


def state_del():
    '''Delete current boid state 

    '''

    pass


def state_move_down():
    '''Move boid state down in the list 

    '''

    pass


def state_move_up():
    '''Move boid state up in the list 

    '''

    pass



import sys
import typing


def bvh(filepath: str = "",
        filter_glob: str = "*.bvh",
        target: typing.Union[int, str] = 'ARMATURE',
        global_scale: float = 1.0,
        frame_start: int = 1,
        use_fps_scale: bool = False,
        update_scene_fps: bool = False,
        update_scene_duration: bool = False,
        use_cyclic: bool = False,
        rotate_mode: typing.Union[int, str] = 'NATIVE',
        axis_forward: typing.Union[int, str] = '-Z',
        axis_up: typing.Union[int, str] = 'Y'):
    '''Load a BVH motion capture file 

    :param filepath: File Path, Filepath used for importing the file 
    :type filepath: str
    :param filter_glob: filter_glob 
    :type filter_glob: str
    :param target: Target, Import target type 
    :type target: typing.Union[int, str]
    :param global_scale: Scale, Scale the BVH by this value 
    :type global_scale: float
    :param frame_start: Start Frame, Starting frame for the animation 
    :type frame_start: int
    :param use_fps_scale: Scale FPS, Scale the framerate from the BVH to the current scenes, otherwise each BVH frame maps directly to a Blender frame 
    :type use_fps_scale: bool
    :param update_scene_fps: Update Scene FPS, Set the scene framerate to that of the BVH file (note that this nullifies the ‘Scale FPS’ option, as the scale will be 1:1) 
    :type update_scene_fps: bool
    :param update_scene_duration: Update Scene Duration, Extend the scene’s duration to the BVH duration (never shortens the scene) 
    :type update_scene_duration: bool
    :param use_cyclic: Loop, Loop the animation playback 
    :type use_cyclic: bool
    :param rotate_mode: Rotation, Rotation conversionQUATERNION Quaternion, Convert rotations to quaternions.NATIVE Euler (Native), Use the rotation order defined in the BVH file.XYZ Euler (XYZ), Convert rotations to euler XYZ.XZY Euler (XZY), Convert rotations to euler XZY.YXZ Euler (YXZ), Convert rotations to euler YXZ.YZX Euler (YZX), Convert rotations to euler YZX.ZXY Euler (ZXY), Convert rotations to euler ZXY.ZYX Euler (ZYX), Convert rotations to euler ZYX. 
    :type rotate_mode: typing.Union[int, str]
    :param axis_forward: Forward 
    :type axis_forward: typing.Union[int, str]
    :param axis_up: Up 
    :type axis_up: typing.Union[int, str]
    '''

    pass



import sys
import typing


def addon_keymap_register(keymap_data):
    '''See ‘blender_defaults.py’ for examples of the format this takes. 

    '''

    pass


def addon_keymap_unregister(keymap_data):
    '''Unregister a set of keymaps for addons. 

    '''

    pass


def keyconfig_test(kc):
    '''

    '''

    pass



import sys
import typing
from . import timers
from . import handlers
from . import translations
from . import icons

alembic = None
'''constant value bpy.app.alembic(supported=True, version=(1, 7, 8), version_string= 1, 7, 8) '''

background = None
'''Boolean, True when blender is running without a user interface (started with -b) '''

binary_path = None
'''The location of Blenders executable, useful for utilities that open new instances '''

build_branch = None
'''The branch this blender instance was built from '''

build_cflags = None
'''C compiler flags '''

build_commit_date = None
'''The date of commit this blender instance was built '''

build_commit_time = None
'''The time of commit this blender instance was built '''

build_commit_timestamp = None
'''The unix timestamp of commit this blender instance was built '''

build_cxxflags = None
'''C++ compiler flags '''

build_date = None
'''The date this blender instance was built '''

build_hash = None
'''The commit hash this blender instance was built with '''

build_linkflags = None
'''Binary linking flags '''

build_options = None
'''constant value bpy.app.build_options(bullet=True, codec_avi=True, codec_ffmpeg=True, codec_sndfile=True, compositor=True, cycles=True, cycles_osl=True, freestyle=True, image_cineon=True, image_dds=True, image_hdr=True, image_openexr=True, image_openjpeg=True, image_tiff=True, input_ndof=True, audaspace=True, international=True, openal=True, sdl=True, sdl_dynload=True, jack=True, libmv=True, mod_fluid=True, mod_oceansim=True, mod_remesh=True, mod_smoke=True, collada=True, opencolorio=True, openmp=True, openvdb=True, …) '''

build_platform = None
'''The platform this blender instance was built for '''

build_system = None
'''Build system used '''

build_time = None
'''The time this blender instance was built '''

build_type = None
'''The type of build (Release, Debug) '''

factory_startup = None
'''Boolean, True when blender is running with factory-startup) '''

ffmpeg = None
'''constant value bpy.app.ffmpeg(supported=True, avcodec_version=(58, 18, 100), avcodec_version_string=‘58, 18, 100, avdevice_version=(58, 3, 100), avdevice_version_string=‘58, 3, 100, avformat_version=(58, 12, 100), avformat_version_string=‘58, 12, 100, avutil_version=(56, 14, 100), avutil_version_string=‘56, 14, 100, swscale_version=(5, 1, 100), swscale_version_string= 5, 1, 100) '''

handlers = None
'''constant value bpy.app.handlers(frame_change_pre=[], frame_change_post=[], render_pre=[], render_post=[], render_write=[], render_stats=[], render_init=[], render_complete=[], render_cancel=[], load_pre=[], load_post=[], save_pre=[], save_post=[], undo_pre=[], undo_post=[], redo_pre=[], redo_post=[], depsgraph_update_pre=[], depsgraph_update_post=[], version_update=[<function do_versions at 0x7f8cfc76eae8>], load_factory_preferences_post=[], load_factory_startup_post=[], persistent=<class ‘persistent>) '''

icons = None
'''Manage custom icons '''

ocio = None
'''constant value bpy.app.ocio(supported=True, version=(1, 1, 0), version_string= 1, 1, 0) '''

oiio = None
'''constant value bpy.app.oiio(supported=True, version=(1, 8, 13), version_string= 1, 8, 13) '''

opensubdiv = None
'''constant value bpy.app.opensubdiv(supported=True, version=(0, 0, 0), version_string= 0, 0, 0) '''

openvdb = None
'''constant value bpy.app.openvdb(supported=True, version=(5, 1, 0), version_string= 5, 1, 0) '''

sdl = None
'''constant value bpy.app.sdl(supported=True, version=(2, 0, 8), version_string=‘2.0.8, available=True) '''

timers = None
'''Manage timers '''

translations = None
'''Application and addons internationalization API '''

version = None
'''The Blender version as a tuple of 3 numbers. eg. (2, 50, 11) '''

version_char = None
'''The Blender version character (for minor releases) '''

version_cycle = None
'''The release status of this build alpha/beta/rc/release '''

version_string = None
'''The Blender version formatted as a string '''



import sys
import typing


def bake():
    '''Bake dynamic paint image sequence surface 

    '''

    pass


def output_toggle(output: typing.Union[int, str] = 'A'):
    '''Add or remove Dynamic Paint output data layer 

    :param output: Output Toggle 
    :type output: typing.Union[int, str]
    '''

    pass


def surface_slot_add():
    '''Add a new Dynamic Paint surface slot 

    '''

    pass


def surface_slot_remove():
    '''Remove the selected surface slot 

    '''

    pass


def type_toggle(type: typing.Union[int, str] = 'CANVAS'):
    '''Toggle whether given type is active or not 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass



import sys
import typing


def gizmo_select(extend: bool = False,
                 deselect: bool = False,
                 toggle: bool = False,
                 deselect_all: bool = False):
    '''Select the currently highlighted gizmo 

    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    :param deselect: Deselect, Remove from selection 
    :type deselect: bool
    :param toggle: Toggle Selection, Toggle the selection 
    :type toggle: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor 
    :type deselect_all: bool
    '''

    pass


def gizmo_tweak():
    '''Tweak the active gizmo 

    '''

    pass



import sys
import typing


def delete():
    '''Delete active scene 

    '''

    pass


def freestyle_add_edge_marks_to_keying_set():
    '''Add the data paths to the Freestyle Edge Mark property of selected edges to the active keying set 

    '''

    pass


def freestyle_add_face_marks_to_keying_set():
    '''Add the data paths to the Freestyle Face Mark property of selected polygons to the active keying set 

    '''

    pass


def freestyle_alpha_modifier_add(
        type: typing.Union[int, str] = 'ALONG_STROKE'):
    '''Add an alpha transparency modifier to the line style associated with the active lineset 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def freestyle_color_modifier_add(
        type: typing.Union[int, str] = 'ALONG_STROKE'):
    '''Add a line color modifier to the line style associated with the active lineset 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def freestyle_fill_range_by_selection(type: typing.Union[int, str] = 'COLOR',
                                      name: str = ""):
    '''Fill the Range Min/Max entries by the min/max distance between selected mesh objects and the source object 

    :param type: Type, Type of the modifier to work onCOLOR Color, Color modifier type.ALPHA Alpha, Alpha modifier type.THICKNESS Thickness, Thickness modifier type. 
    :type type: typing.Union[int, str]
    :param name: Name, Name of the modifier to work on 
    :type name: str
    '''

    pass


def freestyle_geometry_modifier_add(
        type: typing.Union[int, str] = '2D_OFFSET'):
    '''Add a stroke geometry modifier to the line style associated with the active lineset 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def freestyle_lineset_add():
    '''Add a line set into the list of line sets 

    '''

    pass


def freestyle_lineset_copy():
    '''Copy the active line set to a buffer 

    '''

    pass


def freestyle_lineset_move(direction: typing.Union[int, str] = 'UP'):
    '''Change the position of the active line set within the list of line sets 

    :param direction: Direction, Direction to move the active line set towards 
    :type direction: typing.Union[int, str]
    '''

    pass


def freestyle_lineset_paste():
    '''Paste the buffer content to the active line set 

    '''

    pass


def freestyle_lineset_remove():
    '''Remove the active line set from the list of line sets 

    '''

    pass


def freestyle_linestyle_new():
    '''Create a new line style, reusable by multiple line sets 

    '''

    pass


def freestyle_modifier_copy():
    '''Duplicate the modifier within the list of modifiers 

    '''

    pass


def freestyle_modifier_move(direction: typing.Union[int, str] = 'UP'):
    '''Move the modifier within the list of modifiers 

    :param direction: Direction, Direction to move the chosen modifier towards 
    :type direction: typing.Union[int, str]
    '''

    pass


def freestyle_modifier_remove():
    '''Remove the modifier from the list of modifiers 

    '''

    pass


def freestyle_module_add():
    '''Add a style module into the list of modules 

    '''

    pass


def freestyle_module_move(direction: typing.Union[int, str] = 'UP'):
    '''Change the position of the style module within in the list of style modules 

    :param direction: Direction, Direction to move the chosen style module towards 
    :type direction: typing.Union[int, str]
    '''

    pass


def freestyle_module_open(filepath: str = "", make_internal: bool = True):
    '''Open a style module file 

    :param filepath: filepath 
    :type filepath: str
    :param make_internal: Make internal, Make module file internal after loading 
    :type make_internal: bool
    '''

    pass


def freestyle_module_remove():
    '''Remove the style module from the stack 

    '''

    pass


def freestyle_stroke_material_create():
    '''Create Freestyle stroke material for testing 

    '''

    pass


def freestyle_thickness_modifier_add(
        type: typing.Union[int, str] = 'ALONG_STROKE'):
    '''Add a line thickness modifier to the line style associated with the active lineset 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def gpencil_brush_preset_add(name: str = "",
                             remove_name: bool = False,
                             remove_active: bool = False):
    '''Add or remove grease pencil brush preset 

    :param name: Name, Name of the preset, used to make the path name 
    :type name: str
    :param remove_name: remove_name 
    :type remove_name: bool
    :param remove_active: remove_active 
    :type remove_active: bool
    '''

    pass


def gpencil_material_preset_add(name: str = "",
                                remove_name: bool = False,
                                remove_active: bool = False):
    '''Add or remove grease pencil material preset 

    :param name: Name, Name of the preset, used to make the path name 
    :type name: str
    :param remove_name: remove_name 
    :type remove_name: bool
    :param remove_active: remove_active 
    :type remove_active: bool
    '''

    pass


def light_cache_bake(delay: int = 0, subset: typing.Union[int, str] = 'ALL'):
    '''Bake the active view layer lighting 

    :param delay: Delay, Delay in millisecond before baking starts 
    :type delay: int
    :param subset: Subset, Subset of probes to updateALL All LightProbes, Bake both irradiance grids and reflection cubemaps.DIRTY Dirty Only, Only bake lightprobes that are marked as dirty.CUBEMAPS Cubemaps Only, Try to only bake reflection cubemaps if irradiance grids are up to date. 
    :type subset: typing.Union[int, str]
    '''

    pass


def light_cache_free():
    '''Delete cached indirect lighting 

    '''

    pass


def new(type: typing.Union[int, str] = 'NEW'):
    '''Add new scene by type 

    :param type: TypeNEW New, Add a new, empty scene with default settings.EMPTY Copy Settings, Add a new, empty scene, and copy settings from the current scene.LINK_COPY Linked Copy, Link in the collections from the current scene (shallow copy).FULL_COPY Full Copy, Make a full copy of the current scene. 
    :type type: typing.Union[int, str]
    '''

    pass


def render_view_add():
    '''Add a render view 

    '''

    pass


def render_view_remove():
    '''Remove the selected render view 

    '''

    pass


def view_layer_add():
    '''Add a view layer 

    '''

    pass


def view_layer_remove():
    '''Remove the selected view layer 

    '''

    pass



import sys
import typing
import bpy


def edge_face_count(mesh) -> list:
    '''

    :return:  list face users for each item in mesh.edges. 
    '''

    pass


def edge_face_count_dict(mesh) -> dict:
    '''

    :return:  dict of edge keys with their value set to the number of faces using each edge. 
    '''

    pass


def edge_loops_from_edges(mesh, edges=None):
    '''closed loops have matching start and end values. 

    '''

    pass


def mesh_linked_triangles(mesh: 'bpy.types.Mesh') -> list:
    '''Splits the mesh into connected triangles, use this for separating cubes from other mesh elements within 1 mesh datablock. 

    :param mesh: the mesh used to group with. 
    :type mesh: 'bpy.types.Mesh'
    :return:  lists of lists containing triangles. 
    '''

    pass


def mesh_linked_uv_islands(mesh: 'bpy.types.Mesh') -> list:
    '''Splits the mesh into connected polygons, use this for separating cubes from other mesh elements within 1 mesh datablock. 

    :param mesh: the mesh used to group with. 
    :type mesh: 'bpy.types.Mesh'
    :return:  lists of lists containing polygon indices 
    '''

    pass


def ngon_tessellate(from_data: typing.List['bpy.types.Mesh'],
                    indices: list,
                    fix_loops: bool = True,
                    debug_print=True):
    '''Takes a polyline of indices (fgon) and returns a list of face index lists. Designed to be used for importers that need indices for an fgon to create from existing verts. 

    :param from_data: either a mesh, or a list/tuple of vectors. 
    :type from_data: typing.List['bpy.types.Mesh']
    :param indices: a list of indices to use this list is the ordered closed polyline to fill, and can be a subset of the data given. 
    :type indices: list
    :param fix_loops: If this is enabled polylines that use loops to make multiple polylines are delt with correctly. 
    :type fix_loops: bool
    '''

    pass


def triangle_random_points(
        num_points,
        loop_triangles: typing.List['bpy.types.MeshLoopTriangle']) -> list:
    '''Generates a list of random points over mesh loop triangles. 

    :param num_points: the number of random points to generate on each triangle. 
    :param loop_triangles: list of the triangles to generate points on. 
    :type loop_triangles: typing.List['bpy.types.MeshLoopTriangle']
    :return:  list of random points over all triangles. 
    '''

    pass



import sys
import typing
depsgraph_update_post = None
'''on depsgraph update (post) '''

depsgraph_update_pre = None
'''on depsgraph update (pre) '''

frame_change_post = None
'''on frame change for playback and rendering (after) '''

frame_change_pre = None
'''on frame change for playback and rendering (before) '''

load_factory_preferences_post = None
'''on loading factory preferences (after) '''

load_factory_startup_post = None
'''on loading factory startup (after) '''

load_post = None
'''on loading a new blend file (after) '''

load_pre = None
'''on loading a new blend file (before) '''

persistent = None
'''Function decorator for callback functions not to be removed when loading new files '''

redo_post = None
'''on loading a redo step (after) '''

redo_pre = None
'''on loading a redo step (before) '''

render_cancel = None
'''on canceling a render job '''

render_complete = None
'''on completion of render job '''

render_init = None
'''on initialization of a render job '''

render_post = None
'''on render (after) '''

render_pre = None
'''on render (before) '''

render_stats = None
'''on printing render statistics '''

render_write = None
'''on writing a render frame (directly after the frame is written) '''

save_post = None
'''on saving a blend file (after) '''

save_pre = None
'''on saving a blend file (before) '''

undo_post = None
'''on loading an undo step (after) '''

undo_pre = None
'''on loading an undo step (before) '''

version_update = None
'''on ending the versioning code '''



import sys
import typing


def add_point(location: int = (0, 0)):
    '''Add New Paint Curve Point 

    :param location: Location, Location of vertex in area space 
    :type location: int
    '''

    pass


def add_point_slide(PAINTCURVE_OT_add_point=None, PAINTCURVE_OT_slide=None):
    '''Add new curve point and slide it 

    :param PAINTCURVE_OT_add_point: Add New Paint Curve Point, Add New Paint Curve Point 
    :param PAINTCURVE_OT_slide: Slide Paint Curve Point, Select and slide paint curve point 
    '''

    pass


def cursor():
    '''Place cursor 

    '''

    pass


def delete_point():
    '''Remove Paint Curve Point 

    '''

    pass


def draw():
    '''Draw curve 

    '''

    pass


def new():
    '''Add new paint curve 

    '''

    pass


def select(location: int = (0, 0), toggle: bool = False, extend: bool = False):
    '''Select a paint curve point 

    :param location: Location, Location of vertex in area space 
    :type location: int
    :param toggle: Toggle, (De)select all 
    :type toggle: bool
    :param extend: Extend, Extend selection 
    :type extend: bool
    '''

    pass


def slide(align: bool = False, select: bool = True):
    '''Select and slide paint curve point 

    :param align: Align Handles, Aligns opposite point handle during transform 
    :type align: bool
    :param select: Select, Attempt to select a point handle before transform 
    :type select: bool
    '''

    pass



import sys
import typing


def autopack_toggle():
    '''Automatically pack all external files into the .blend file 

    '''

    pass


def bookmark_add():
    '''Add a bookmark for the selected/active directory 

    '''

    pass


def bookmark_cleanup():
    '''Delete all invalid bookmarks 

    '''

    pass


def bookmark_delete(index: int = -1):
    '''Delete selected bookmark 

    :param index: Index 
    :type index: int
    '''

    pass


def bookmark_move(direction: typing.Union[int, str] = 'TOP'):
    '''Move the active bookmark up/down in the list 

    :param direction: Direction, Direction to move the active bookmark towardsTOP Top, Top of the list.UP Up.DOWN Down.BOTTOM Bottom, Bottom of the list. 
    :type direction: typing.Union[int, str]
    '''

    pass


def bookmark_toggle():
    '''Toggle bookmarks display 

    '''

    pass


def cancel():
    '''Cancel loading of selected file 

    '''

    pass


def delete():
    '''Delete selected files 

    '''

    pass


def directory_new(directory: str = "", open: bool = False):
    '''Create a new directory 

    :param directory: Directory, Name of new directory 
    :type directory: str
    :param open: Open, Open new directory 
    :type open: bool
    '''

    pass


def execute(need_active: bool = False):
    '''Execute selected file 

    :param need_active: Need Active, Only execute if there’s an active selected file in the file list 
    :type need_active: bool
    '''

    pass


def filenum(increment: int = 1):
    '''Increment number in filename 

    :param increment: Increment 
    :type increment: int
    '''

    pass


def filepath_drop(filepath="Path"):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def find_missing_files(
        find_all: bool = False,
        directory: str = "",
        filter_blender: bool = False,
        filter_backup: bool = False,
        filter_image: bool = False,
        filter_movie: bool = False,
        filter_python: bool = False,
        filter_font: bool = False,
        filter_sound: bool = False,
        filter_text: bool = False,
        filter_btx: bool = False,
        filter_collada: bool = False,
        filter_alembic: bool = False,
        filter_folder: bool = False,
        filter_blenlib: bool = False,
        filemode: int = 9,
        display_type: typing.Union[int, str] = 'DEFAULT',
        sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA'):
    '''Try to find missing external files 

    :param find_all: Find All, Find all files in the search path (not just missing) 
    :type find_all: bool
    :param directory: Directory, Directory of the file 
    :type directory: str
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    '''

    pass


def hidedot():
    '''Toggle hide hidden dot files 

    '''

    pass


def highlight():
    '''Highlight selected file(s) 

    '''

    pass


def make_paths_absolute():
    '''Make all paths to external files absolute 

    '''

    pass


def make_paths_relative():
    '''Make all paths to external files relative to current .blend 

    '''

    pass


def next():
    '''Move to next folder 

    '''

    pass


def pack_all():
    '''Pack all used external files into the .blend 

    '''

    pass


def pack_libraries():
    '''Pack all used Blender library files into the current .blend 

    '''

    pass


def parent():
    '''Move to parent directory 

    '''

    pass


def previous():
    '''Move to previous folder 

    '''

    pass


def refresh():
    '''Refresh the file list 

    '''

    pass


def rename():
    '''Rename file or file directory 

    '''

    pass


def report_missing_files():
    '''Report all missing external files 

    '''

    pass


def reset_recent():
    '''Reset Recent files 

    '''

    pass


def select(extend: bool = False, fill: bool = False, open: bool = True):
    '''Activate/select file 

    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    :param fill: Fill, Select everything beginning with the last selection 
    :type fill: bool
    :param open: Open, Open a directory when selecting it 
    :type open: bool
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Select or deselect all files 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_bookmark(dir: str = ""):
    '''Select a bookmarked directory 

    :param dir: Dir 
    :type dir: str
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET'):
    '''Activate/select the file(s) contained in the border 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def select_walk(direction: typing.Union[int, str] = 'UP',
                extend: bool = False,
                fill: bool = False):
    '''Select/Deselect files by walking through them 

    :param direction: Walk Direction, Select/Deselect file in this direction 
    :type direction: typing.Union[int, str]
    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    :param fill: Fill, Select everything beginning with the last selection 
    :type fill: bool
    '''

    pass


def smoothscroll():
    '''Smooth scroll to make editable file visible 

    '''

    pass


def unpack_all(method: typing.Union[int, str] = 'USE_LOCAL'):
    '''Unpack all files packed into this .blend to external ones 

    :param method: Method, How to unpack 
    :type method: typing.Union[int, str]
    '''

    pass


def unpack_item(method: typing.Union[int, str] = 'USE_LOCAL',
                id_name: str = "",
                id_type: int = 19785):
    '''Unpack this file to an external file 

    :param method: Method, How to unpack 
    :type method: typing.Union[int, str]
    :param id_name: ID name, Name of ID block to unpack 
    :type id_name: str
    :param id_type: ID Type, Identifier type of ID block 
    :type id_type: int
    '''

    pass


def unpack_libraries():
    '''Unpack all used Blender library files from this .blend file 

    '''

    pass



import sys
import typing
import mathutils


class BVHTree:
    def find_nearest(self, origin, distance: float = 1.84467e+19):
        '''Find the nearest element (typically face index) to a point. 

        :param co: Find nearest element to this point. 
        :type co: 'mathutils.Vector'
        :param distance: Maximum distance threshold. 
        :type distance: float
        :return:  Returns a tuple (Vector location, Vector normal, int index, float distance), Values will all be None if no hit is found. 
        '''
        pass

    def find_nearest_range(self, origin,
                           distance: float = 1.84467e+19) -> list:
        '''Find the nearest elements (typically face index) to a point in the distance range. 

        :param co: Find nearest elements to this point. 
        :type co: 'mathutils.Vector'
        :param distance: Maximum distance threshold. 
        :type distance: float
        :rtype: list
        :return:  Returns a list of tuples (Vector location, Vector normal, int index, float distance), 
        '''
        pass

    def overlap(self, other_tree: 'BVHTree') -> list:
        '''Find overlapping indices between 2 trees. 

        :param other_tree: Other tree to perform overlap test on. 
        :type other_tree: 'BVHTree'
        :rtype: list
        :return:  Returns a list of unique index pairs, the first index referencing this tree, the second referencing the other_tree. 
        '''
        pass

    def ray_cast(self,
                 origin,
                 direction: 'mathutils.Vector',
                 distance: float = sys.float_info.max):
        '''Cast a ray onto the mesh. 

        :param co: Start location of the ray in object space. 
        :type co: 'mathutils.Vector'
        :param direction: Direction of the ray in object space. 
        :type direction: 'mathutils.Vector'
        :param distance: Maximum distance threshold. 
        :type distance: float
        :return:  Returns a tuple (Vector location, Vector normal, int index, float distance), Values will all be None if no hit is found. 
        '''
        pass

    def __init__(self, size):
        '''

        '''
        pass



import sys
import typing


def denoise_animation(input_filepath: str = "", output_filepath: str = ""):
    '''Denoise rendered animation sequence using current scene and view layer settings. Requires denoising data passes and output to OpenEXR multilayer files 

    :param input_filepath: Input Filepath, File path for image to denoise. If not specified, uses the render file path and frame range from the scene 
    :type input_filepath: str
    :param output_filepath: Output Filepath, If not specified, renders will be denoised in-place 
    :type output_filepath: str
    '''

    pass


def merge_images(input_filepath1: str = "",
                 input_filepath2: str = "",
                 output_filepath: str = ""):
    '''Combine OpenEXR multilayer images rendered with different sampleranges into one image with reduced noise 

    :param input_filepath1: Input Filepath, File path for image to merge 
    :type input_filepath1: str
    :param input_filepath2: Input Filepath, File path for image to merge 
    :type input_filepath2: str
    :param output_filepath: Output Filepath, File path for merged image 
    :type output_filepath: str
    '''

    pass


def use_shading_nodes():
    '''Enable nodes on a material, world or light 

    '''

    pass



import sys
import typing
import bpy


def addon_disable(module: str = ""):
    '''Disable an add-on 

    :param module: Module, Module name of the add-on to disable 
    :type module: str
    '''

    pass


def addon_enable(module: str = ""):
    '''Enable an add-on 

    :param module: Module, Module name of the add-on to enable 
    :type module: str
    '''

    pass


def addon_expand(module: str = ""):
    '''Display information and preferences for this add-on 

    :param module: Module, Module name of the add-on to expand 
    :type module: str
    '''

    pass


def addon_install(overwrite: bool = True,
                  target: typing.Union[int, str] = 'DEFAULT',
                  filepath: str = "",
                  filter_folder: bool = True,
                  filter_python: bool = True,
                  filter_glob: str = "*.py;*.zip"):
    '''Install an add-on 

    :param overwrite: Overwrite, Remove existing add-ons with the same ID 
    :type overwrite: bool
    :param target: Target Path 
    :type target: typing.Union[int, str]
    :param filepath: filepath 
    :type filepath: str
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_python: Filter python 
    :type filter_python: bool
    :param filter_glob: filter_glob 
    :type filter_glob: str
    '''

    pass


def addon_refresh():
    '''Scan add-on directories for new modules 

    '''

    pass


def addon_remove(module: str = ""):
    '''Delete the add-on from the file system 

    :param module: Module, Module name of the add-on to remove 
    :type module: str
    '''

    pass


def addon_show(module: str = ""):
    '''Show add-on preferences 

    :param module: Module, Module name of the add-on to expand 
    :type module: str
    '''

    pass


def app_template_install(overwrite: bool = True,
                         filepath: str = "",
                         filter_folder: bool = True,
                         filter_glob: str = "*.zip"):
    '''Install an application-template 

    :param overwrite: Overwrite, Remove existing template with the same ID 
    :type overwrite: bool
    :param filepath: filepath 
    :type filepath: str
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_glob: filter_glob 
    :type filter_glob: str
    '''

    pass


def copy_prev():
    '''Copy settings from previous version 

    '''

    pass


def keyconfig_activate(filepath: str = ""):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param filepath: filepath 
    :type filepath: str
    '''

    pass


def keyconfig_export(all: bool = False,
                     filepath: str = "keymap.py",
                     filter_folder: bool = True,
                     filter_text: bool = True,
                     filter_python: bool = True):
    '''Export key configuration to a python script 

    :param all: All Keymaps, Write all keymaps (not just user modified) 
    :type all: bool
    :param filepath: filepath 
    :type filepath: str
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_text: Filter text 
    :type filter_text: bool
    :param filter_python: Filter python 
    :type filter_python: bool
    '''

    pass


def keyconfig_import(filepath: str = "keymap.py",
                     filter_folder: bool = True,
                     filter_text: bool = True,
                     filter_python: bool = True,
                     keep_original: bool = True):
    '''Import key configuration from a python script 

    :param filepath: filepath 
    :type filepath: str
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_text: Filter text 
    :type filter_text: bool
    :param filter_python: Filter python 
    :type filter_python: bool
    :param keep_original: Keep original, Keep original file after copying to configuration folder 
    :type keep_original: bool
    '''

    pass


def keyconfig_remove():
    '''Remove key config 

    '''

    pass


def keyconfig_test():
    '''Test key-config for conflicts 

    '''

    pass


def keyitem_add():
    '''Add key map item 

    '''

    pass


def keyitem_remove(item_id: int = 0):
    '''Remove key map item 

    :param item_id: Item Identifier, Identifier of the item to remove 
    :type item_id: int
    '''

    pass


def keyitem_restore(item_id: int = 0):
    '''Restore key map item 

    :param item_id: Item Identifier, Identifier of the item to remove 
    :type item_id: int
    '''

    pass


def keymap_restore(all: bool = False):
    '''Restore key map(s) 

    :param all: All Keymaps, Restore all keymaps to default 
    :type all: bool
    '''

    pass


def reset_default_theme():
    '''Reset to the default theme colors 

    '''

    pass


def studiolight_copy_settings(index: int = 0):
    '''Copy Studio Light settings to the Studio light editor 

    :param index: index 
    :type index: int
    '''

    pass


def studiolight_install(
        files: typing.List['bpy.types.OperatorFileListElement'] = None,
        directory: str = "",
        filter_folder: bool = True,
        filter_glob: str = "*.png;*.jpg;*.hdr;*.exr",
        type: typing.Union[int, str] = 'MATCAP'):
    '''Install a user defined studio light 

    :param files: File Path 
    :type files: typing.List['bpy.types.OperatorFileListElement']
    :param directory: directory 
    :type directory: str
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_glob: filter_glob 
    :type filter_glob: str
    :param type: type 
    :type type: typing.Union[int, str]
    '''

    pass


def studiolight_new(filename: str = "StudioLight"):
    '''Save custom studio light from the studio light editor settings 

    :param filename: Name 
    :type filename: str
    '''

    pass


def studiolight_show():
    '''Show light preferences 

    '''

    pass


def studiolight_uninstall(index: int = 0):
    '''Delete Studio Light 

    :param index: index 
    :type index: int
    '''

    pass


def theme_install(overwrite: bool = True,
                  filepath: str = "",
                  filter_folder: bool = True,
                  filter_glob: str = "*.xml"):
    '''Load and apply a Blender XML theme file 

    :param overwrite: Overwrite, Remove existing theme file if exists 
    :type overwrite: bool
    :param filepath: filepath 
    :type filepath: str
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_glob: filter_glob 
    :type filter_glob: str
    '''

    pass



import sys
import typing
contexts = None
'''constant value bpy.app.translations.contexts(default_real=None, default=*, operator_default=Operator, ui_events_keymaps=UI_Events_KeyMaps, plural=Plural, id_action=Action, id_armature=Armature, id_brush=Brush, id_camera=Camera, id_cachefile=CacheFile, id_collection=Collection, id_curve=Curve, id_fs_linestyle=FreestyleLineStyle, id_gpencil=GPencil, id_id=ID, id_image=Image, id_shapekey=Key, id_light=Light, id_library=Library, id_lattice=Lattice, id_mask=Mask, …) '''

contexts_C_to_py = None
'''A readonly dict mapping contexts C-identifiers to their py-identifiers. '''


def locale_explode(locale):
    '''For non-complete locales, missing elements will be None. 

    :param locale: The ISO locale string to explode. 
    :return:  A tuple (language, country, variant, language_country, language@variant). 
    '''

    pass


def pgettext(msgid: str, msgctxt: str):
    '''Try to translate the given msgid (with optional msgctxt). 

    :param msgid: The string to translate. 
    :type msgid: str
    :param msgctxt: The translation context (defaults to BLT_I18NCONTEXT_DEFAULT). 
    :type msgctxt: str
    :return:  The translated string (or msgid if no translation was found). 
    '''

    pass


def pgettext_data(msgid: str, msgctxt: str):
    '''Try to translate the given msgid (with optional msgctxt), if new data name’s translation is enabled. 

    :param msgid: The string to translate. 
    :type msgid: str
    :param msgctxt: The translation context (defaults to BLT_I18NCONTEXT_DEFAULT). 
    :type msgctxt: str
    :return:  The translated string (or msgid if no translation was found). 
    '''

    pass


def pgettext_iface(msgid: str, msgctxt: str):
    '''Try to translate the given msgid (with optional msgctxt), if labels’ translation is enabled. 

    :param msgid: The string to translate. 
    :type msgid: str
    :param msgctxt: The translation context (defaults to BLT_I18NCONTEXT_DEFAULT). 
    :type msgctxt: str
    :return:  The translated string (or msgid if no translation was found). 
    '''

    pass


def pgettext_tip(msgid: str, msgctxt: str):
    '''Try to translate the given msgid (with optional msgctxt), if tooltips’ translation is enabled. 

    :param msgid: The string to translate. 
    :type msgid: str
    :param msgctxt: The translation context (defaults to BLT_I18NCONTEXT_DEFAULT). 
    :type msgctxt: str
    :return:  The translated string (or msgid if no translation was found). 
    '''

    pass


def register(module_name: str, translations_dict: dict):
    '''Registers an addon’s UI translations. 

    :param module_name: The name identifying the addon. 
    :type module_name: str
    :param translations_dict: A dictionary built like that: {locale: {msg_key: msg_translation, ...}, ...} 
    :type translations_dict: dict
    '''

    pass


def unregister(module_name: str):
    '''Unregisters an addon’s UI translations. 

    :param module_name: The name identifying the addon. 
    :type module_name: str
    '''

    pass



import sys
import typing


def is_registered(function: int) -> bool:
    '''Check if this function is registered as a timer. 

    :param function: Function to check. 
    :type function: int
    :return:  True when this function is registered, otherwise False. 
    '''

    pass


def register(function, first_interval: float = 0, persistent: bool = False):
    '''Add a new function that will be called after the specified amount of seconds. The function gets no arguments and is expected to return either None or a float. If None is returned, the timer will be unregistered. A returned number specifies the delay until the function is called again. functools.partial can be used to assign some parameters. 

    :param function: The function that should called. 
    :param first_interval: Seconds until the callback should be called the first time. 
    :type first_interval: float
    :param persistent: Don’t remove timer when a new file is loaded. 
    :type persistent: bool
    '''

    pass


def unregister(function):
    '''Unregister timer. 

    :param function: Function to unregister. 
    '''

    pass



import sys
import typing
categories = None
'''constant value bpy.utils.units.categories(NONE=NONE, LENGTH=LENGTH, AREA=AREA, VOLUME=VOLUME, MASS=MASS, ROTATION=ROTATION, TIME=TIME, VELOCITY=VELOCITY, ACCELERATION=ACCELERATION, CAMERA=CAMERA, POWER=POWER) '''

systems = None
'''constant value bpy.utils.units.systems(NONE=NONE, METRIC=METRIC, IMPERIAL=IMPERIAL) '''


def to_string(unit_system: str,
              unit_category: str,
              value: float,
              precision: int = 3,
              split_unit: bool = False,
              compatible_unit: bool = False) -> str:
    '''Convert a given input float value into a string with units. 

    :param unit_system: The unit system, from bpy.utils.units.systems. 
    :type unit_system: str
    :param unit_category: The category of data we are converting (length, area, rotation, etc.), from bpy.utils.units.categories. 
    :type unit_category: str
    :param value: The value to convert to a string. 
    :type value: float
    :param precision: Number of digits after the comma. 
    :type precision: int
    :param split_unit: Whether to use several units if needed (1m1cm), or always only one (1.01m). 
    :type split_unit: bool
    :param compatible_unit: Whether to use keyboard-friendly units (1m2) or nicer utf-8 ones (1m²). 
    :type compatible_unit: bool
    :return:  The converted string. 
    '''

    pass


def to_value(unit_system: str,
             unit_category: str,
             str_input: str,
             str_ref_unit: str = None) -> float:
    '''Convert a given input string into a float value. 

    :param unit_system: The unit system, from bpy.utils.units.systems. 
    :type unit_system: str
    :param unit_category: The category of data we are converting (length, area, rotation, etc.), from bpy.utils.units.categories. 
    :type unit_category: str
    :param str_input: The string to convert to a float value. 
    :type str_input: str
    :param str_ref_unit: A reference string from which to extract a default unit, if none is found in str_input. 
    :type str_ref_unit: str
    :return:  The converted/interpreted value. 
    '''

    pass



import sys
import typing
import bpy


def bake_action(obj: 'bpy.types.Object', *, action: 'bpy.types.Action',
                frames: int, **kwargs) -> 'bpy.types.Action':
    '''

    :param obj: Object to bake. 
    :type obj: 'bpy.types.Object'
    :param action: An action to bake the data into, or None for a new action to be created. 
    :type action: 'bpy.types.Action'
    :param frames: Frames to bake. 
    :type frames: int
    :return:  an action or None 
    '''

    pass


def bake_action_iter(obj: 'bpy.types.Object',
                     *,
                     action: 'bpy.types.Action',
                     only_selected: bool = False,
                     do_pose: bool = True,
                     do_object: bool = True,
                     do_visual_keying: bool = True,
                     do_constraint_clear: bool = False,
                     do_parents_clear: bool = False,
                     do_clean: bool = False) -> 'bpy.types.Action':
    '''An coroutine that bakes action for a single object. 

    :param obj: Object to bake. 
    :type obj: 'bpy.types.Object'
    :param action: An action to bake the data into, or None for a new action to be created. 
    :type action: 'bpy.types.Action'
    :param only_selected: Only bake selected bones. 
    :type only_selected: bool
    :param do_pose: Bake pose channels. 
    :type do_pose: bool
    :param do_object: Bake objects. 
    :type do_object: bool
    :param do_visual_keying: Use the final transformations for baking (‘visual keying’) 
    :type do_visual_keying: bool
    :param do_constraint_clear: Remove constraints after baking. 
    :type do_constraint_clear: bool
    :param do_parents_clear: Unparent after baking objects. 
    :type do_parents_clear: bool
    :param do_clean: Remove redundant keyframes after baking. 
    :type do_clean: bool
    :return:  an action or None 
    '''

    pass


def bake_action_objects(object_action_pairs, *, frames: int,
                        **kwargs) -> typing.List['bpy.types.Action']:
    '''A version of bake_action_objects_iter() that takes frames and returns the output. 

    :param frames: Frames to bake. 
    :type frames: int
    :return:  A sequence of Action or None types (aligned with object_action_pairs) 
    '''

    pass


def bake_action_objects_iter(object_action_pairs: 'bpy.types.Sequence',
                             **kwargs):
    '''An coroutine that bakes actions for multiple objects. 

    :param object_action_pairs: Sequence of object action tuples, action is the destination for the baked data. When None a new action will be created. 
    :type object_action_pairs: 'bpy.types.Sequence'
    '''

    pass



import sys
import typing


def autocomplete():
    '''Show a list of used text in the open document 

    '''

    pass


def comment():
    '''Convert selected text to comment 

    '''

    pass


def convert_whitespace(type: typing.Union[int, str] = 'SPACES'):
    '''Convert whitespaces by type 

    :param type: Type, Type of whitespace to convert to 
    :type type: typing.Union[int, str]
    '''

    pass


def copy():
    '''Copy selected text to clipboard 

    '''

    pass


def cursor_set(x: int = 0, y: int = 0):
    '''Set cursor position 

    :param x: X 
    :type x: int
    :param y: Y 
    :type y: int
    '''

    pass


def cut():
    '''Cut selected text to clipboard 

    '''

    pass


def delete(type: typing.Union[int, str] = 'NEXT_CHARACTER'):
    '''Delete text by cursor position 

    :param type: Type, Which part of the text to delete 
    :type type: typing.Union[int, str]
    '''

    pass


def duplicate_line():
    '''Duplicate the current line 

    '''

    pass


def find():
    '''Find specified text 

    '''

    pass


def find_set_selected():
    '''Find specified text and set as selected 

    '''

    pass


def indent():
    '''Indent selected text 

    '''

    pass


def insert(text: str = ""):
    '''Insert text at cursor position 

    :param text: Text, Text to insert at the cursor position 
    :type text: str
    '''

    pass


def jump(line: int = 1):
    '''Jump cursor to line 

    :param line: Line, Line number to jump to 
    :type line: int
    '''

    pass


def line_break():
    '''Insert line break at cursor position 

    '''

    pass


def line_number():
    '''The current line number 

    '''

    pass


def make_internal():
    '''Make active text file internal 

    '''

    pass


def move(type: typing.Union[int, str] = 'LINE_BEGIN'):
    '''Move cursor to position type 

    :param type: Type, Where to move cursor to 
    :type type: typing.Union[int, str]
    '''

    pass


def move_lines(direction: typing.Union[int, str] = 'DOWN'):
    '''Move the currently selected line(s) up/down 

    :param direction: Direction 
    :type direction: typing.Union[int, str]
    '''

    pass


def move_select(type: typing.Union[int, str] = 'LINE_BEGIN'):
    '''Move the cursor while selecting 

    :param type: Type, Where to move cursor to, to make a selection 
    :type type: typing.Union[int, str]
    '''

    pass


def new():
    '''Create a new text data-block 

    '''

    pass


def open(filepath: str = "",
         filter_blender: bool = False,
         filter_backup: bool = False,
         filter_image: bool = False,
         filter_movie: bool = False,
         filter_python: bool = True,
         filter_font: bool = False,
         filter_sound: bool = False,
         filter_text: bool = True,
         filter_btx: bool = False,
         filter_collada: bool = False,
         filter_alembic: bool = False,
         filter_folder: bool = True,
         filter_blenlib: bool = False,
         filemode: int = 9,
         display_type: typing.Union[int, str] = 'DEFAULT',
         sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA',
         internal: bool = False):
    '''Open a new text data-block 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    :param internal: Make internal, Make text file internal after loading 
    :type internal: bool
    '''

    pass


def overwrite_toggle():
    '''Toggle overwrite while typing 

    '''

    pass


def paste(selection: bool = False):
    '''Paste text from clipboard 

    :param selection: Selection, Paste text selected elsewhere rather than copied (X11 only) 
    :type selection: bool
    '''

    pass


def refresh_pyconstraints():
    '''Refresh all pyconstraints 

    '''

    pass


def reload():
    '''Reload active text data-block from its file 

    '''

    pass


def replace():
    '''Replace text with the specified text 

    '''

    pass


def replace_set_selected():
    '''Replace text with specified text and set as selected 

    '''

    pass


def resolve_conflict(resolution: typing.Union[int, str] = 'IGNORE'):
    '''When external text is out of sync, resolve the conflict 

    :param resolution: Resolution, How to solve conflict due to differences in internal and external text 
    :type resolution: typing.Union[int, str]
    '''

    pass


def run_script():
    '''Run active script 

    '''

    pass


def save():
    '''Save active text data-block 

    '''

    pass


def save_as(filepath: str = "",
            check_existing: bool = True,
            filter_blender: bool = False,
            filter_backup: bool = False,
            filter_image: bool = False,
            filter_movie: bool = False,
            filter_python: bool = True,
            filter_font: bool = False,
            filter_sound: bool = False,
            filter_text: bool = True,
            filter_btx: bool = False,
            filter_collada: bool = False,
            filter_alembic: bool = False,
            filter_folder: bool = True,
            filter_blenlib: bool = False,
            filemode: int = 9,
            display_type: typing.Union[int, str] = 'DEFAULT',
            sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA'):
    '''Save active text file with options 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files 
    :type check_existing: bool
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    '''

    pass


def scroll(lines: int = 1):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param lines: Lines, Number of lines to scroll 
    :type lines: int
    '''

    pass


def scroll_bar(lines: int = 1):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param lines: Lines, Number of lines to scroll 
    :type lines: int
    '''

    pass


def select_all():
    '''Select all text 

    '''

    pass


def select_line():
    '''Select text by line 

    '''

    pass


def select_word():
    '''Select word under cursor 

    '''

    pass


def selection_set(select: bool = False):
    '''Set cursor selection 

    :param select: Select, Set selection end rather than cursor 
    :type select: bool
    '''

    pass


def start_find():
    '''Start searching text 

    '''

    pass


def to_3d_object(split_lines: bool = False):
    '''Create 3D text object from active text data-block 

    :param split_lines: Split Lines, Create one object per line in the text 
    :type split_lines: bool
    '''

    pass


def uncomment():
    '''Convert selected comment to text 

    '''

    pass


def unindent():
    '''Unindent selected text 

    '''

    pass


def unlink():
    '''Unlink active text data-block 

    '''

    pass
